@ECHO OFF

::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Uso:
:: runclient [PARAMS] SERVIDOR_CSBASE[:PORTA] [SERVIDOR_DTD]
::
:: Parmetros:
:: [-h | --help]    ... (opcional)    Imprime uma mensagem com o manual do 'runclient' e retorna. 
:: [-P | --pause]   ... (opcional)    Para ao final da execucao. 
:: | --port] PORTA  ... (opcional)    Define a porta utilizada pelo cliente para exportar objetos. 
:: SERVIDOR[:PORTA] ... (obrigatrio) Domnio e porta onde o servidor exportou seu ponto de entrada.
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Inicializacao dos parametros.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
set pause=false
set clientport=0
set serveraddr=

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Lendo os argumentos passados.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:while
  IF [%1]==[] GOTO elihw
   
  IF "%1" EQU "-p" SET clientport=%2 & SHIFT & GOTO continue
  IF "%1" EQU "--port" SET clientport=%2 & SHIFT & GOTO continue

  IF "%1" EQU "-h" GOTO usage
  IF "%1" EQU "--help" GOTO usage

  IF "%1" EQU "-P" SET pause=true & GOTO continue
  IF "%1" EQU "--pause" SET pause=true & GOTO continue

  SET serveraddr=%1
  GOTO elihw  
  
  :continue
  SHIFT
  GOTO while
:elihw

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::  Inicializando o cliente csbase.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

IF [%serveraddr%]==[] (
  ECHO Nao foi fornecido um servidor a ser conectado. Assumindo localhost...
  SET serveraddr=localhost
)

SETLOCAL ENABLEDELAYEDEXPANSION
SET SEPARATOR=;
SET JAR_PATH=
set CSBASE_DIR=..
set CSBASE_LIBS=%CSBASE_DIR%\csbase\libs

SET DIRS=( ^
      javamail ^
      jaf ^
      jfreechart ^
      svg ^
      javautils ^
      csdk ^
      pdfrenderer ^
      javawebstart ^
      ftc ^
      miglayout ^
)

FOR %%D IN %DIRS% DO FOR %%J IN (%CSBASE_LIBS%\%%D\*.jar) DO SET JAR_PATH=!JAR_PATH!%SEPARATOR%%%J
SET JAR_PATH=%JAR_PATH:~1%

java -Dsun.swing.enableImprovedDragGesture -Dfile.encoding=ISO-8859-1 -classpath .;%CSBASE_DIR%;%CSBASE_DIR%\csbase;%JAR_PATH% csbase.client.ClientLauncher "--server_url csbase://%serveraddr%" "--locale pt_BR" "--rmi_port %clientport%"

IF %pause%==true (
  ECHO.
  ECHO Pressione ENTER para fechar a janela...
  PAUSE >NUL
)
:: Vou ao final do script, ao invez de usar o comando EXIT, para evitar fechar o prompt.
GOTO end


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Imprime uma mensagem com o manual do runclient e retorna.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:usage

CLS
ECHO.
ECHO NOME
ECHO   runclient - Inicializa o cliente do CSGrid conectando-o a um servidor.
ECHO.
ECHO USOS
ECHO   runclient (-h ^| --help)
ECHO   runclient [OPCOES] [SERVIDOR[:PORTA]]
ECHO.
ECHO OPCOES
ECHO   -h, --help
ECHO     Imprime uma mensagem com o manual do 'runclient' e retorna sem inicializar o cliente.
ECHO.
ECHO   -P, --pause
ECHO     Para ao final da execucao.
ECHO.   
ECHO   -p PORTA, --port PORTA
ECHO     Permite definir qual PORTA o cliente ir utilizar para exportar seus objetos para o servidor.
ECHO     Se a porta ja estiver sendo utilizada, ou se o servidor nao conseguir acessar aquela porta do cliente, o cliente fara polling no servidor.
ECHO     O valor 0 indica que o sistema deve definir uma porta para ser utilizada.
ECHO     Um valor negativo indica que o cliente deve fazer polling no servidor.
ECHO     Na ausencia deste parametro, sera utilizado o valor padrao 0.
ECHO.
ECHO EXEMPLOS
ECHO   runclient
ECHO     Inicia o cliente conectando-o ao servidor padrao localhost na porta padrao 1099.
ECHO     Indica que o cliente deve escolher uma porta livre para exportar seus objetos.
ECHO     Caso o servidor no consiga acessar o cliente na porta escolhida, o cliente ira fazer polling no servidor.
ECHO.
ECHO   runclient -p 0
ECHO     Inicia o cliente conectando-o ao servidor padrao localhost na porta padrao 1099.
ECHO     Indica que o cliente deve escolher uma porta livre para exportar seus objetos.
ECHO     Caso o servidor no consiga acessar o cliente na porta escolhida, o cliente ira fazer polling no servidor.
ECHO.
ECHO   runclient -p 1096
ECHO     Inicia o cliente conectando-o ao servidor padrao localhost na porta padrao 1099.
ECHO     Indica o cliente para utilizar a porta 1096 para exportar seus objetos.
ECHO     Caso a porta j esteja em uso, ou o servidor no consiga acessar o cliente na porta escolhida, o cliente ira fazer polling no servidor.
ECHO.
ECHO   runclient -p -1
ECHO     Inicia o cliente conectando-o ao servidor padrao localhost na porta padrao 1099.
ECHO     Indica que o cliente deve fazer polling no servidor.
ECHO.
ECHO   runclient almada
ECHO     Inicia o cliente conectando-o ao servidor almada na porta padrao 1099.
ECHO     Indica que o cliente deve escolher uma porta livre para exportar seus objetos.
ECHO     Caso o servidor no consiga acessar o cliente na porta escolhida, o cliente ira fazer polling no servidor.
ECHO.
ECHO   runclient -p 1096 almada:1097
ECHO     Inicia o cliente conectando-o ao servidor almada escutando na porta 1097.
ECHO     Indica o cliente para utilizar a porta 1096 para exportar seus objetos.
ECHO     Caso a porta j esteja em uso, ou o servidor no consiga acessar o cliente na porta escolhida, o cliente ira fazer polling no servidor.
ECHO.
ECHO   runclient -p -1 almada:1097
ECHO     Inicia o cliente conectando-o ao servidor almada na porta 1097.
ECHO     Indica que o cliente deve fazer polling no servidor.
ECHO.


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Termina o programa, pausando se assim foi requisitado.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:end
